# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.2.0

import enum

import winrt.system
from winrt import _winrt_windows_storage_fileproperties

__all__ = [
    "PhotoOrientation",
    "PropertyPrefetchOptions",
    "ThumbnailMode",
    "ThumbnailOptions",
    "ThumbnailType",
    "VideoOrientation",
    "BasicProperties",
    "DocumentProperties",
    "GeotagHelper",
    "ImageProperties",
    "MusicProperties",
    "StorageItemContentProperties",
    "StorageItemThumbnail",
    "VideoProperties",
    "IStorageItemExtraProperties",
]

class PhotoOrientation(enum.IntEnum):
    UNSPECIFIED = 0
    NORMAL = 1
    FLIP_HORIZONTAL = 2
    ROTATE180 = 3
    FLIP_VERTICAL = 4
    TRANSPOSE = 5
    ROTATE270 = 6
    TRANSVERSE = 7
    ROTATE90 = 8

class PropertyPrefetchOptions(enum.IntFlag):
    NONE = 0x0
    MUSIC_PROPERTIES = 0x1
    VIDEO_PROPERTIES = 0x2
    IMAGE_PROPERTIES = 0x4
    DOCUMENT_PROPERTIES = 0x8
    BASIC_PROPERTIES = 0x10

class ThumbnailMode(enum.IntEnum):
    PICTURES_VIEW = 0
    VIDEOS_VIEW = 1
    MUSIC_VIEW = 2
    DOCUMENTS_VIEW = 3
    LIST_VIEW = 4
    SINGLE_ITEM = 5

class ThumbnailOptions(enum.IntFlag):
    NONE = 0x0
    RETURN_ONLY_IF_CACHED = 0x1
    RESIZE_THUMBNAIL = 0x2
    USE_CURRENT_SCALE = 0x4

class ThumbnailType(enum.IntEnum):
    IMAGE = 0
    ICON = 1

class VideoOrientation(enum.IntEnum):
    NORMAL = 0
    ROTATE90 = 90
    ROTATE180 = 180
    ROTATE270 = 270

BasicProperties = _winrt_windows_storage_fileproperties.BasicProperties
DocumentProperties = _winrt_windows_storage_fileproperties.DocumentProperties
GeotagHelper = _winrt_windows_storage_fileproperties.GeotagHelper
ImageProperties = _winrt_windows_storage_fileproperties.ImageProperties
MusicProperties = _winrt_windows_storage_fileproperties.MusicProperties
StorageItemContentProperties = _winrt_windows_storage_fileproperties.StorageItemContentProperties
StorageItemThumbnail = _winrt_windows_storage_fileproperties.StorageItemThumbnail
VideoProperties = _winrt_windows_storage_fileproperties.VideoProperties
IStorageItemExtraProperties = _winrt_windows_storage_fileproperties.IStorageItemExtraProperties
